;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  	System Hacker 1.7                        	               /
;/                              Written by Donald Straney 8/18/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

#include "Ti86.inc"						;Standard include file
#include "syshackd.inc"					;Need the addresses of data in external file

;Vars
Cur_State		equ		$ca00	;1		;Passed to event handler - current mode
Key_Val			equ		$ca01	;1		;Passed from event handler - current keypress
Link_Val		equ		$ca02	;1		;Passed from event handler - current link port state
Link_Prev		equ		$ca03	;1		;Used in event handler - previous value on link port
Ret_Addr		equ		$ca04	;2		;Passed to event handler - address to return to when "exit" pressed
Top_Addr		equ		$ca06	;2		;Used in hex, edit, asm modes - first displayed address
Bot_Addr		equ		$ca08	;2		;Used in hex, edit, asm modes - last displayed address
Cursor_Addr		equ		$ca0a	;2		;Used in edit mode - address in buffer of byte selected by cursor
Cursor_Pos		equ		$ca0c	;2		;Used in edit mode - position of cursor saved here
Saved_Pos		equ		$ca0e	;1		;Used in edit mode - 1 if position saved in Cursor_Pos
ROM_Page		equ		$ca0f	;1		;Used in hex, edit, asm modes - ROM page currently viewed
RAM_Page		equ		$ca10	;1		;Used in hex, edit, asm modes - RAM page currently viewed
Char_Num		equ		$ca11	;1		;Used in char submode - number of current char
Port_Num		equ		$ca12	;1		;Used in port submode - number of current port
Link_Rx_Pos		equ		$ca13	;2		;Used in link mode - current positions of cursor for input and output data
Link_Tx_Pos		equ		$ca15	;2
VAT_Top_Addr	equ		$ca17	;2		;Used in VAT submode - pointer to beginning of first viewed entry
VAT_Bot_Addr	equ		$ca19	;2		;Used in VAT submode - pointer to end of last viewed entry
Addr_SP			equ		$ca1b	;2		;Used in asm mode - sp for address stack
VAT_Addr_SP		equ		$ca1d	;2		;Used in VAT submode - sp for VAT address stack
edit_buffer		equ		$ca1f	;6		;Used by input routine - buffer for string
Data_Buffer		equ		$ca25	;48		;Start of buffer where data is copied to be processed
Addr_Stack		equ		$ca25+48		;Used in asm mode - addresses saved here
VAT_Addr_Stack	equ		$ca25+48+200	;Used in VAT submode - VAT addresses saved here

Str_Buf			equ		$c386			;Start of buffer where search string data is parsed
Main_Flags		equ		asm_flag1		;Flags for event handler
Link_Flags		equ		asm_flag2		;Entry and display mode flags for link com mode
Str_Copy_Addr	equ		$8d00			;Location on RAM page 1 where string is copied
Str_Decomp_Addr	equ		$8100			;Location on RAM page 1 where string is decompressed

bitAlpha		equ		0				;In Link_Flags - 1 if sending letter, 0 if sending number or other char
bitHex			equ		1				;In Link_Flags - 1 if chars entered/displayed in hex, 0 if chars
bitNibble		equ		2				;In Link_Flags - 1 if on second hex digit, 0 if on first

bitEnKey		equ		0				;In Main_Flags - 1 if event handler should handle preset keys (f1-f5, exit)

;Constants
Hex_Cols		equ		6				;Number of columns and rows of data displayed in hex, edit modes
Hex_Rows		equ		8
Buffer_Len		equ	Hex_Cols*Hex_Rows	;Length of data to copy to Data_Buffer
Line_Spacing	equ		6				;Number of pixels between lines
Hex_Spacing		equ		2				;Number of pixels between hex bytes (hex, edit mode)
Char_Spacing	equ		7				;Number of pixels between chars (hex, edit mode)
Stack_Spacing	equ		40				;Number of pixels between stack viewer columns (edit mode)
Win_Start_Addr	equ		$fc00+($0e*16)+($18/8)	;Starting address of window drawn by window routine
Win_Width		equ		8				;Size of window drawn by window routine
Win_Height		equ		Line_Spacing*6+1
Timeout_Num		equ		200				;Timeout for link port receive
Max_Search_Len	equ		7				;Max number of bytes in search string
Link_Lines		equ		3				;Max number of lines of text for each link text window

;Cursor positions
Top_Bar_Pos		equ		$0000			;Position of top bar
Bot_Bar_Pos		equ		$3900			;Position of bottom bar
Hex_Start_Pos	equ		$0715			;Position where hex is displayed (hex, edit modes)
Char_Start_Pos	equ		$0756			;Position where chars are displayed (hex, edit modes)
RAM_Pos			equ		$0053			;Position where RAM page is displayed
ROM_Pos			equ		$0069			;Position where ROM page is displayed
Window_Pos		equ		$0f1a			;Position for text in window
Input_Pos		equ		$0f1a			;Position of input text
Text_Start_Pos	equ		$0700			;Position of text, such as help text, drawn in window
Undel_Buf_Pos	equ		$3200			;Position where contents of undelete buffer are drawn
Char_Num_Pos	equ		$1e0c			;Position where char code is drawn
Buf_Text_Pos	equ		$2c00			;Position where undelete buffer label is drawn
Port_In_Pos		equ		$1e00			;Positions where input from port is drawn
Port_In_2_Pos	equ		$1e0a
Port_Label_Pos	equ		$3200			;Position where port description is drawn
Link_In_Pos		equ		$2000			;Starting positions of link in and out data
Link_Out_Pos	equ		$0700
Flag_Start_Pos	equ		$077b			;Starting position where states of flags are displayed

Char_Control_Pos	equ	$0102			;Position of controls for char mode
Char_Pos		equ		$0202
Port_Control_Pos	equ	$0102			;Position of controls for port mode
Port_Pos		equ		$0202

Hex_Left		equ		$15				;Left edge of hex data
Hex_Right		equ		Hex_Left+((Hex_Cols-1)*(Hex_Spacing+8))	;Right edge of hex data
Hex_Top			equ		7				;Top edge of hex data
Hex_Bot			equ		7+((Hex_Rows-1)*Line_Spacing)		;Bottom edge of hex data
Edit_Bot_Pos	equ		Hex_Bot*256+Hex_Left

;States in Cur_State
stMenu	equ		0						;In menu routine
stHelp	equ		1						;Help mode
stHex	equ		2						;Hex mode
stEdit	equ		3						;Edit mode
stAsm	equ		4						;Asm mode
stPort	equ		5						;Port submode
stChar	equ		6						;Char submode
stLink	equ		7						;Link submode
stVAT	equ		8						;VAT submode
stStack	equ		9						;Stack viewer submode
stGetkey	equ	10						;_getkey code submode
stGetky	equ		11						;_getky code submode
stBASkey	equ	12						;BASIC keycode submode

.org _asm_exec_ram

	nop
	jp Prog_Start

	.dw 0,Shell_Title

Shell_Title:
	.db "System Hacker 1.7",0

Prog_Start:
	call _runindicoff					;Normal init
	call _flushallmenus
	call _clrLCD
	ld hl,Str_Name-1					;Load string with data
	rst 20h
	rst 10h
	ret c								;Don't run if string isn't found
	call _ex_ahl_bde					;If it does exist, load onto RAM page 1
	call _ahl_plus_2_pg3
	call _set_abs_src_addr
	ld a,$01							;Set destination address
	ld hl,Str_Copy_Addr-$8000
	call _set_abs_dest_addr
	ld hl,Str_Name-1
	rst 20h
	rst 10h
    call _get_size_word
    xor a
    ex de,hl
    call _set_mm_num_bytes              ;Set number of bytes for copy to size of string
	call _mm_ldir						;Copy string
	ld hl,Str_Copy_Addr					;Now decompress it
	ld de,Str_Decomp_Addr
	call Decompress
	in a,(7)							;Init vars
	ld (Link_Prev),a
	set bitEnKey,(iy+Main_Flags)
	xor a
	ld (Saved_Pos),a
	ld (Char_Num),a
	ld (Port_Num),a
	ld (DisFlags),a
	ld h,a
	ld l,a
	ld (Top_Addr),hl
	inc a
	ld (RAM_Page),a
	ld a,$0d
	ld (ROM_Page),a
	set textInverse,(iy+textflags)		;Draw the top and bottom bars
	set textEraseBelow,(iy+textflags)
	ld hl,$fc00
	ld b,16*7
Draw_Bar:
	ld (hl),$ff
	inc hl
	djnz Draw_Bar
	ld hl,Top_Bar_Pos
	ld (_penCol),hl
	ld hl,Top_Bar_Str
	call _vputs
	ld hl,Bot_Bar_Pos
	ld (_penCol),hl
	ld hl,Bot_Bar_Str
	call _vputs
	res textInverse,(iy+textflags)
	call Refresh_Page_Disp				;Show what ROM/RAM pages are loaded
	call Hex_Init						;Now go to hex mode

Event_Handler:
	call _getky							;Any key events?
	or a
	jr nz,Key_Event						;If so, go to proper key handler
	in a,(3)							;Is on pressed?
	and $08
	jr nz,Check_Link_Event
	xor a								;If so, set keycode as 0
	ld (Key_Val),a
	jp Pass_Key_To_Handler				;and notify program
Check_Link_Event:
	in a,(7)							;Check for link port activity
	ld b,a
	ld a,(Link_Prev)
	cp b
	jr z,Event_Handler					;If the value on the link port hasn't changed, loop again
Link_Event:
	ld a,b
	ld (Link_Val),a						;Store the link port value for the handler
	ld a,(Cur_State)					;Right now, the link com mode is the only mode that handles link port activity, so go to that
	cp stLink
	call z,Link_Port_Handler
	jr Event_Handler					;If it isn't in that mode, we can ignore link port events, so go back to the event handler loop
Key_Event:
	ld (Key_Val),a						;Pass on key code to key handler for correct mode
	bit bitEnKey,(iy+Main_Flags)		;Is the event handler allowed to "interpret" keys?
	jr z,Pass_Key_To_Handler			;If not, skip next section
	cp K_F1								;If it's any of the mode keys, go to the correct mode
	jr nz,Test_Keys_0
	call Hex_Init
	jr Event_Handler
Test_Keys_0:
	cp K_F2
	jr nz,Test_Keys_1
	call Edit_Init
	jr Event_Handler
Test_Keys_1:
	cp K_F3
	jr nz,Test_Keys_2
	call Asm_Init
	jr Event_Handler
Test_Keys_2:
	cp K_F4
	jr nz,Test_Keys_3
	call Sys_Init
	jr Event_Handler
Test_Keys_3:
	cp K_F5
	jr nz,Test_Keys_4
	ld a,(Cur_State)					;Don't go into help mode if in menu
	or a
	call nz,Help_Init
	jr Event_Handler
Test_Keys_4:
	cp K_EXIT							;If it's the exit key,
	jr nz,Pass_Key_To_Handler
Exit_Pressed:
	ld a,(Cur_State)					;Now look at state
	cp stPort							;If it's a system submode,
	jr nc,Call_Sys_Init					;go back to system mode
	cp stHex							;If it's a main mode,
	jr nc,Exit_Prog_2					;exit
Call_Ret_Addr:
	ld hl,(Ret_Addr)
	jp Call_HL							;otherwise, call (Ret_Addr)
Call_Sys_Init:
	call Sys_Init
	jp Event_Handler
Pass_Key_To_Handler:
	ld a,(Cur_State)					;Get current state
	ld hl,Key_Handler_Addr_Table		;Get correct position in table
	call Get_Addr_From_Table			;Now get address from table
	jp Call_HL							;and go to it

Exit_Prog:
	pop hl
Exit_Prog_2:
	res 4,(iy+9)						;Make sure OS doesn't think on is pressed
	ld a,$c0							;and reset link port state
	out (7),a
	jp _clrScrn

;//////////////////////////////////////////HEX MODE/////////////////////////////////////////////////

Hex_Init:
	ld a,stHex							;Set state
	ld (Cur_State),a
	ld hl,Hex_Init						;Set to return to hex mode if exit is pressed in menu
	ld (Ret_Addr),hl
	jp Hex_Refresh						;Refresh data on screen

Hex_Key_Handler:
	ld a,(Key_Val)
	cp K_DOWN							;This should be REALLY obvious...
	jr z,Hex_Go_Down
	cp K_UP
	jr z,Hex_Go_Up
	cp K_LEFT
	jr z,Hex_Go_Left
	cp K_RIGHT
	jr z,Hex_Go_Right
	cp K_MINUS
	jr z,Hex_Go_Up_5
	cp K_STAR
	jr z,Hex_Go_Down_10
	cp K_SLASH
	jr z,Hex_Go_Up_10
	cp K_MORE
	jp z,Show_Option_Menu
	cp K_PLUS
	ret nz
Hex_Go_Down_5:
	ld hl,(Bot_Addr)
	ld bc,Buffer_Len*4+1
	add hl,bc
	ld (Top_Addr),hl
	jr Hex_Refresh
Hex_Go_Up_5:
	ld hl,(Top_Addr)
	ld bc,Buffer_Len*5
	sbc hl,bc
	ld (Top_Addr),hl
	jr Hex_Refresh
Hex_Go_Left:
	ld hl,(Top_Addr)
	dec hl
	ld (Top_Addr),hl
	jr Hex_Refresh
Hex_Go_Right:
	ld hl,(Top_Addr)
	inc hl
	ld (Top_Addr),hl
	jr Hex_Refresh
Hex_Go_Down_10:
	ld hl,(Bot_Addr)
	ld bc,Buffer_Len*9+1
	add hl,bc
	ld (Top_Addr),hl
	jr Hex_Refresh
Hex_Go_Up_10:
	ld hl,(Top_Addr)
	ld bc,Buffer_Len*10
	sbc hl,bc
	ld (Top_Addr),hl
	jr Hex_Refresh
Hex_Go_Down:
	ld hl,(Bot_Addr)					;Go one byte down from the last byte on the screen to get the next page
	inc hl
	ld (Top_Addr),hl
	jr Hex_Refresh
Hex_Go_Up:
	ld hl,(Top_Addr)					;Subtract the number of bytes on the screen to get previous page
	ld bc,Buffer_Len
	sbc hl,bc
	ld (Top_Addr),hl
Hex_Refresh:							;Draw contents of memory on screen
	call Clear_Wind						;Clear screen for displaying data
	call Copy_To_Buffer					;Get data in Data_Buffer
;Draw hex portion of screen
	ld hl,Hex_Start_Pos					;then start cursor in correct position for drawing hex
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,Data_Buffer					;Start at beginning of buffer
	ld b,Hex_Rows						;Init loop counter (number of rows to draw)
Draw_Hex_Rows:
	push bc
	ld a,(_penCol)
	push af
	push hl
	or a
	ld bc,Data_Buffer					;Get offset in buffer, then add top address to get source address
	sbc hl,bc
	ld bc,(Top_Addr)
	add hl,bc
	xor a
	ld (_penCol),a
	call Disp_HL_Hex					;At the beginning of the line, draw the address
	pop hl
	pop af
	ld (_penCol),a
	ld b,Hex_Cols						;Init loop counter (number of cols to draw)
Draw_Hex_Row_Loop:						;Draw a row of data
	ld a,(hl)							;Draw each hex byte
	call Draw_Hex_Byte
	inc hl								;Point to next byte
	ld a,(_penCol)
	add a,Hex_Spacing					;Move cursor to next position
	ld (_penCol),a
	djnz Draw_Hex_Row_Loop				;Loop until the right number of columns have been drawn
	call _vnewline						;When that row is finished, move to the next line
	pop bc
	djnz Draw_Hex_Rows					;Loop until the right number of rows have been drawn
;Now draw chars
	ld hl,Char_Start_Pos				;Start cursor in correct position for drawing chars
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,Data_Buffer					;and again, as any idiot could figure out, start at beginning of buffer
	ld b,Hex_Rows						;Same as before, just draw each char instead of its hex representation...
Draw_Char_Rows:
	push bc
	ld b,Hex_Cols
Draw_Char_Row_Loop:
	ld a,(_penCol)
	push af
	ld a,(hl)
	or a
	call z,Draw_Unprintable
	cp $e7
	call nc,Draw_Unprintable
	cp $d6
	call z,Draw_Unprintable_2
	call _vputmap
Draw_Char_Row_Skip:
	inc hl
	pop af
	add a,Char_Spacing
	ld (_penCol),a
	djnz Draw_Char_Row_Loop
	call _vnewline
	pop bc
	djnz Draw_Char_Rows
	ret

Draw_Unprintable:
	ld a,$d0
	ret
Draw_Unprintable_2:
	ld a,$3a
	ret

;/////////////////////////////////////////OPTION MENU//////////////////////////////////////////////

Show_Option_Menu:
	call Draw_Window					;Draw window
	ld hl,Window_Pos					;Position cursor
	ld (_penCol),hl
	ld hl,Option_Menu_Text				;then draw menu and wait for keypress
	ld de,Option_Menu_Table
	ld b,Option_Menu_Num
	jp Do_Menu

Option_Addr:
	ld a,4								;(require 4 digits)
	ld hl,Addr_Prompt_Str				;(set prompt string)
	ld b,1
	call Get_Hex_Num					;Get the address where the user wants to go
	or a								;Did the user enter a relative address?
	jr z,Option_Addr_End
	dec a								;If so, is it a +?
	jr z,Option_Addr_F
Option_Addr_B:							;If not, it must be a -
	ex de,hl
	ld hl,(Top_Addr)
	sbc hl,de							;If it was a -, go back by the number of bytes the user specified
	jr Option_Addr_End
Option_Addr_F:
	ex de,hl
	ld hl,(Top_Addr)
	add hl,de							;If it was a +, go forward by the number of bytes the user specified
Option_Addr_End:
	ld (Top_Addr),hl					;Set the address to the one the user entered
	xor a								;If in edit mode, make sure that the cursor starts at the top
	ld (Saved_Pos),a
	jp Go_Back_To_Prev_State

Option_Abs:
	ld a,6								;Set to get 6 digits
	ld hl,Abs_Prompt_Str				;Set prompt
	ld b,0								;No relative addressing!
	call Get_Hex_Num					;Get address
	ld a,h								;put in ahl
	ld h,l
	ld l,b
	call _conv_ahl						;and decode
	ld (Top_Addr),hl					;Now set address
	ld (RAM_Page),a
	call Refresh_Page_Disp				;show what page we're on
	xor a
	ld (Saved_Pos),a
	jp Go_Back_To_Prev_State			;and return

Option_Search:
	call Draw_Window					;Clear window
	ld hl,Input_Pos						;Position cursor
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,Search_Prompt_Str				;Display prompt
	call _vputs_newline
	ld hl,Str_Buf
	ld b,2*Max_Search_Len				;(4 bytes max)
	call TextIn							;Get string
	jp nz,Go_Back_To_Prev_State			;Cancel if exit pressed
	ld a,2*Max_Search_Len				;Get length of string
	sub b
	ld b,a
	rra
	and %01111111
	rla
	cp b								;Is the length even (2 digits/byte)?
	jp nz,Value_Error					;If not, show error
	rra									;If so continue - divide length by 2 to get number of bytes
	and %01111111
	ld (Patch8),a						;then save length
	ld hl,Str_Buf						;Now parse string
	push hl
	pop de
	ld b,a								;Init loop counter with number of bytes
Parse_Loop:
	ld a,(de)
	call Convert_Char
	call _shlacc
	ld (hl),a
	inc de
	ld a,(de)
	call Convert_Char
	or (hl)
	ld (hl),a
	inc hl
	inc de
	djnz Parse_Loop
	call Draw_Window					;Show "searching..." message
	ld hl,Window_Pos
	ld (_penCol),hl
	ld hl,Searching_Str
	call _vputs
Patch8	equ	$+1
	ld a,0								;Restore data length
	ld (size),a							;and save it for search routine
	ld hl,(Top_Addr)					;See where we are
	ld e,a								;Make sure that we're moving on to the NEXT occurrence of the string...
	ld d,0
	add hl,de
	ld de,$4000
	call _cphlde
	jr c,Search_ROM_0
	ld de,$8000
	call _cphlde
	jr nc,Test_More_Addr
	ld a,(ROM_Page)
	ld (page),a
	jr Search_ROM
Test_More_Addr:
	ld de,$c000
	call _cphlde
	jr nc,Search_RAM_0
	ld a,(RAM_Page)
	or $40
	ld (page),a
	jr Search_RAM

Search_ROM_0:							;Search ROM page 0
	ld de,$4000
	call Calc_Search_Length
	call Search
	jp nc,Data_Found
	ld hl,$4000
	ld a,1
	ld (page),a
Search_ROM:								;Search other ROM pages
	ld de,$8000
	call Calc_Search_Length
Search_ROM_Loop:
	ld a,(page)
	out (5),a
	inc a
	ld (page),a
	cp $10
	jr z,Search_ROM_Done
	call Search
	jr nc,Data_Found_ROM
	ld hl,$4000
	ld bc,$4000
	jr Search_ROM_Loop
Search_ROM_Done:
	ld a,$41
	ld (page),a
	ld hl,$8000
Search_RAM:								;Search swapped RAM pages
	ld de,$c000
	call Calc_Search_Length
Search_RAM_Loop:
	ld a,(page)
	out (6),a
	inc a
	ld (page),a
	cp $48
	jr z,Search_RAM_Done
	call Search
	jr nc,Data_Found_RAM
	ld hl,$8000
	jr Search_RAM_Loop
Search_RAM_Done:
	ld hl,$c000
Search_RAM_0:							;Search RAM page 0
	ld de,$ffff
	call Calc_Search_Length
	call Search
	jr nc,Data_Found
	call Reset_Pages
	jp Not_Found_Error					;If it wasn't found, show error
Data_Found_RAM:							;If it was found, go to the correct page
	ld a,(page)
	dec a
	and $0f
	ld (RAM_Page),a
	jr Data_Found
Data_Found_ROM:
	ld a,(page)
	dec a
	ld (ROM_Page),a
Data_Found:
	ld (Top_Addr),hl					;and the correct address
	call Reset_Pages
	call Refresh_Page_Disp
	xor a
	ld (Saved_Pos),a
	jp Go_Back_To_Prev_State

Option_Var:
	call Draw_Window					;Erase current window
	ld hl,Input_Pos						;Position cursor
	ld (_penCol),hl
	ld hl,Var_Prompt_Str				;Display prompt
	call _vputs
	ld hl,_OP1+2
	ld b,8
	call TextIn							;Get var name
	jp nz,Go_Back_To_Prev_State			;Cancel if exit was pressed
	ld a,8
	sub b
	ld (_OP1+1),a						;(set name length)
	rst 10h								;Find var
	call Reset_Pages
	jp c,Not_Found_Error				;If it doesn't exist, show error
	call _ex_ahl_bde
	call _conv_ahl						;get the address and the page it's on
	ld (RAM_Page),a
	ld (Top_Addr),hl
	call Refresh_Page_Disp				;show which page we're on
	jp Go_Back_To_Prev_State			;and go back and show var

Option_RAM:
	ld a,1								;Only need 1 digit
	ld hl,RAM_Page_Prompt_Str			;Set prompt string
	ld b,0
	call Get_Hex_Num					;Get page number
	ld a,h
	call _shracc
	cp 8								;If it's invalid, show error
	jp nc,Value_Error
	or a								;If we want page 0, we don't need to swap in a different page
	jr z,Go_To_RAM0
	ld (RAM_Page),a						;Otherwise, set the page
	ld hl,$8000							;address
	ld (Top_Addr),hl
	call Refresh_Page_Disp
	jp Go_Back_To_Prev_State			;and then go to that address
Go_To_RAM0:
	ld hl,$c000
	ld (Top_Addr),hl
	jp Go_Back_To_Prev_State

Option_ROM:
	ld a,1
	ld hl,ROM_Page_Prompt_Str
	ld b,0
	call Get_Hex_Num
	ld a,h
	call _shracc
	or a
	jr z,Go_To_ROM0
	ld (ROM_Page),a
	ld hl,$4000
	ld (Top_Addr),hl
	call Refresh_Page_Disp
	jp Go_Back_To_Prev_State
Go_To_ROM0:
	ld hl,$0000
	ld (Top_Addr),hl
	jp Go_Back_To_Prev_State

;/////////////////////////////////////////EDIT MODE/////////////////////////////////////////////////

Edit_Init:
	xor a
	ld (Saved_Pos),a
Edit_Init_2:
	ld a,stEdit							;Set state
	ld (Cur_State),a
	ld hl,Edit_Init_2					;Set to return to edit mode if exit is pressed in menu
	ld (Ret_Addr),hl
	jp Edit_Refresh						;Refresh data on screen

Edit_Key_Handler:
	ld a,(Key_Val)
	cp K_DOWN							;This should be REALLY obvious...
	jr z,Edit_Go_Down
	cp K_UP
	jr z,Edit_Go_Up
	cp K_RIGHT
	jr z,Edit_Go_Right
	cp K_LEFT
	jr z,Edit_Go_Left
	cp K_SECOND
	jp z,Edit_Edit
	cp K_MORE
	ret nz
	call Save_Cursor_Pos				;If drawing option menu, save cursor position first
	jp Show_Option_Menu
Edit_Go_Down:
	ld a,(_penRow)						;Get the y position of the cursor
	cp Hex_Bot							;Is it at the bottom of the screen already?
	jp z,Edit_Go_Down_Page				;If so, move down one page
	push af
	call Draw_Cur						;Otherwise, erase old cursor
	pop af
	add a,Line_Spacing					;move one line down on screen
	ld (_penRow),a
	ld hl,(Cursor_Addr)					;move one line down in buffer
	ld bc,Hex_Cols
	add hl,bc
	ld (Cursor_Addr),hl
	jp Draw_Edit_Cursor					;and redraw cursor
Edit_Go_Up:
	ld a,(_penRow)						;Get the y position of the cursor
	cp Hex_Top							;Is it at the top of the screen already?
	jp z,Edit_Go_Up_Page				;If so, move up one page
	push af
	call Draw_Cur						;Otherwise, erase old cursor
	pop af
	sub Line_Spacing					;move one line up on screen
	ld (_penRow),a
	ld hl,(Cursor_Addr)					;move one line up in buffer
	ld bc,Hex_Cols
	sbc hl,bc
	ld (Cursor_Addr),hl
	jp Draw_Edit_Cursor					;and redraw cursor
Edit_Go_Right:
	ld a,(_penCol)						;Get the x position of the cursor
	cp Hex_Right						;Is it at the right of the screen already?
	ret z								;If so, don't let it move!!!!!!
	push af
	call Draw_Cur						;Otherwise, erase old cursor
	pop af
	add a,Hex_Spacing+8				;move one column over on screen
	ld (_penCol),a
	ld hl,(Cursor_Addr)					;move one column over in buffer
	inc hl
	ld (Cursor_Addr),hl
	jp Draw_Edit_Cursor					;and redraw cursor
Edit_Go_Left:
	ld a,(_penCol)						;Get the x position of the cursor
	cp Hex_Left							;Is it at the left of the screen already?
	ret z								;If so, don't let it move
	push af
	call Draw_Cur						;Otherwise, erase old cursor
	pop af
	sub Hex_Spacing+8					;move one column over on screen
	ld (_penCol),a
	ld hl,(Cursor_Addr)					;move one column over in buffer
	dec hl
	ld (Cursor_Addr),hl
	jp Draw_Edit_Cursor					;and redraw cursor
Edit_Edit:
	call Save_Cursor_Pos
	ld hl,(Cursor_Addr)					;Find the actual location in memory we'll be editing
	ld bc,Data_Buffer
	sbc hl,bc
	ld bc,(Top_Addr)
	add hl,bc
	ld (Patch5),hl						;Save the address
	ld de,$8000							;Check to see if we're trying to edit the ROM
	call _cphlde
	jr nc,Edit_Get_Num
	ld hl,Input_Pos						;Otherwise, display message for idiot user
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	call Draw_Window
	ld hl,ROM_Edit_Str
	call _vputs_newline
	call _vputs
	call Pause							;Wait for a keypress
	jr Edit_Refresh						;then draw back screen
Edit_Get_Num:
	ld a,2								;Get new value
	ld hl,Value_Prompt_Str
	ld b,0
	call Get_Hex_Num
	ld c,h
	ld a,(RAM_Page)						;load correct page
	set 6,a
	out (6),a
Patch5	equ	$+1
	ld hl,$0000							;and put it at the proper address
	ld (hl),c	
	call Reset_Pages
	jr Edit_Refresh						;then refresh screen
Edit_Go_Down_Page:
	xor a								;Make sure that cursor starts at beginning
	ld (Saved_Pos),a
	ld hl,(Bot_Addr)					;Go one byte down from the last byte on the screen to get the next page
	inc hl
	ld (Top_Addr),hl
	jr Edit_Refresh
Edit_Go_Up_Page:
	ld hl,Edit_Bot_Pos					;Cursor starts at bottom of screen
	ld (Cursor_Pos),hl
	ld a,h
	ld (Saved_Pos),a
	ld hl,Data_Buffer+((Hex_Rows-1)*Hex_Cols)
	ld (Cursor_Addr),hl
	ld hl,(Top_Addr)					;Subtract the number of bytes on the screen to get previous page
	ld bc,Buffer_Len
	sbc hl,bc
	ld (Top_Addr),hl
Edit_Refresh:
	call Hex_Refresh					;Draw data from memory
	ld a,(Saved_Pos)					;Is there a saved cursor position?
	or a
	jr z,Init_Edit_Cursor
	ld hl,(Cursor_Pos)					;If so, restore it
	ld (_penCol),hl
	jr Draw_Edit_Cursor
Init_Edit_Cursor:						;Set pointer and position for edit cursor at beginning of buffer
	ld hl,Data_Buffer					;Start at beginning of buffer
	ld (Cursor_Addr),hl
	ld hl,Hex_Start_Pos					;and at the upper left corner of the screen
	ld (_penCol),hl
Draw_Edit_Cursor:						;Draw edit cursor at (_penCol),(_penRow), with byte pointed to by (Cursor_Addr)
	set textInverse,(iy+textflags)		;Draw cursor in inverse text
	call Draw_Cur
	res textInverse,(iy+textflags)
	ret
Draw_Cur:
	ld hl,(_penCol)						;Save cursor position
	push hl
	ld hl,(Cursor_Addr)					;Get byte that should be drawn
	ld a,(hl)
	call Draw_Hex_Byte					;and draw it
	pop hl								;Restore cursor position
	ld (_penCol),hl
	ret

Save_Cursor_Pos:
	ld hl,(_penCol)						;If drawing option menu, save cursor position first
	ld (Cursor_Pos),hl
	ld a,1
	ld (Saved_Pos),a
	ret

;////////////////////////////////////////////HELP///////////////////////////////////////////////////

Help_Init:
	ld a,(Cur_State)					;Look at previous state so that we can display the appropriate help text
	cp stHelp							;Wait a minute, is it trying to display help for the help?
	ret z								;We can't allow that!!!!  Back to the event handler you go!!!
	push af
	ld a,stHelp							;Set state
	ld (Cur_State),a
	ld hl,Text_Start_Pos				;Set cursor position
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	pop af								;Now draw appropriate text
	ld hl,Help_Text_Table
	call Get_Addr_From_Table
	ld b,(hl)							;First get length byte
	inc hl
	push bc
	push hl
	call Clear_Wind						;then clear window
	ld hl,Help_Str						;then draw always-present help strings
	call _vputs_newline
	call _vputs_newline
	call _vputs_newline
	pop hl
	pop bc
Help_Draw_Loop:
	call _vputs_newline					;then draw as many lines of text as are needed
	djnz Help_Draw_Loop
	ret

Help_Key_Handler:
	ld a,(Key_Val)
	cp K_SECOND							;If it's second or exit, go back to previous state
	jp nz,Go_Back_To_Prev_State
Help_Show_Credits:
	call Clear_Wind						;Clear window
	ld hl,Text_Start_Pos				;position cursor
	ld (_penCol),hl
	ld hl,Credits_Text					;and draw credits
	ld b,Credits_Text_Len
Help_Credits_Loop:
	call _vputs_newline
	djnz Help_Credits_Loop
	ret

;//////////////////////////////////////////ASM MODE/////////////////////////////////////////////////

Asm_Init:
	ld hl,Addr_Stack					;Init address stack
	ld (Addr_SP),hl
Asm_Init_2:
	ld a,stAsm							;Set state
	ld (Cur_State),a
	ld hl,Asm_Init_2					;Set to return here after exiting menu
	ld (Ret_Addr),hl
	jr Asm_Refresh						;then disassemble

Asm_Key_Handler:
	ld a,(Key_Val)						;Yet again, really obvious...
	cp K_MORE
	jp z,Show_Option_Menu
	cp K_RIGHT
	jr z,Asm_Go_Down_Byte
	cp K_LEFT
	jr z,Asm_Go_Up_Byte
	cp K_UP
	jr z,Asm_Go_Up
	cp K_DOWN
	ret nz
Asm_Go_Down:
	ld hl,(Top_Addr)					;Save address
	ld de,(Addr_SP)
	ld a,l
	ld (de),a
	inc de
	ld a,h
	ld (de),a
	inc de
	ld (Addr_SP),de
	ld hl,(Bot_Addr)					;then move down one page, starting where this page left off
	ld (Top_Addr),hl
	jr Asm_Refresh
Asm_Go_Up:
	ld de,(Addr_SP)
	ld hl,Addr_Stack
	call _cphlde						;Is it at the beginning of the stack?
	jr z,Asm_Go_Up_Byte					;If it is, there aren't any saved addresses-we have to go up one byte at a time
	dec de								;Otherwise, pop address
	ld a,(de)
	ld h,a
	dec de
	ld a,(de)
	ld l,a
	ld (Top_Addr),hl					;and go to it
	ld (Addr_SP),de
	jr Asm_Refresh
Asm_Go_Down_Byte:
	ld hl,(Top_Addr)
	inc hl
	ld (Top_Addr),hl
	jr Asm_Refresh
Asm_Go_Up_Byte:
	ld hl,(Top_Addr)
	dec hl
	ld (Top_Addr),hl
Asm_Refresh:
	call Clear_Wind						;First clear window
	ld hl,Text_Start_Pos				;then init cursor
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld a,(ROM_Page)						;then load correct ROM page
	out (5),a
	ld a,(RAM_Page)
	set 6,a
	out (6),a
	ld de,(Top_Addr)					;Start disassembling from beginning
	ld b,Hex_Rows						;Set the number of rows to disassemble
Asm_Refresh_Loop:
	push bc
	call DisLine						;Disassemble a line at the current address
	pop bc
	call _vnewline						;then move to the next line
	djnz Asm_Refresh_Loop				;Do this as many times as needed
	ld (Bot_Addr),de					;We have the bottom address, so store that
	jp Reset_Pages						;Now load the default ROM/RAM pages again
										;I think we're done now...wow...that was easy...

;//////////////////////////////////////////SYS MODE/////////////////////////////////////////////////

Sys_Init:
	ld hl,Exit_Prog						;Set routine to call if exit is pressed
	ld (Ret_Addr),hl
	call Clear_Wind						;Clear window
	ld hl,Text_Start_Pos				;set cursor position
	ld (_penCol),hl
	ld b,Sys_Menu_Num					;and show system menu
	ld hl,Sys_Menu_Text
	ld de,Sys_Menu_Table
	jp Do_Menu

Sys_2_Init:								;Second set of system menus
	ld hl,Exit_Prog
	ld (Ret_Addr),hl
	call Clear_Wind
	ld hl,Text_Start_Pos
	ld (_penCol),hl
	ld b,Sys_2_Menu_Num
	ld hl,Sys_2_Menu_Text
	ld de,Sys_2_Menu_Table
	jp Do_Menu

;Char map
Sys_Chars:
	ld a,stChar							;Set state
	ld (Cur_State),a
	ld hl,Sys_Chars						;Set to return to sys mode if exit pressed
	ld (Ret_Addr),hl
	call Clear_Wind						;Clear window
	ld hl,Char_Control_Pos				;Draw controls
	ld (_curRow),hl
	ld hl,Char_Control_Str
	call _puts
	ld hl,Char_Pos						;Set large cursor back to position where char will be drawn
	ld (_curRow),hl
	ld hl,Buf_Text_Pos
	ld (_penCol),hl
	ld hl,Undel_Buf_Str
	call _vputs
	call Char_Refresh_Buf				;Draw contents of undelete buffer
	jr Char_Refresh_Char				;Draw appropriate char

Char_Key_Handler:
	ld hl,Char_Num						;Set up pointer
	ld a,(Key_Val)
	cp K_RIGHT
	jr z,Char_Inc
	cp K_LEFT
	jr z,Char_Dec
	cp K_SECOND
	jr z,Char_Copy
	cp K_CLEAR
	ret nz
Char_Clear:								;If clear has been pressed, show that there are no chars in the buffer
	xor a
	ld (_undelBufLen),a
	jr Sys_Chars
Char_Inc:
	inc (hl)
	jr Char_Refresh_Char
Char_Dec:
	dec (hl)
Char_Refresh_Char:
	ld hl,Char_Num_Pos					;Set cursor position
	ld (_penCol),hl
	ld a,(Char_Num)						;First draw number
	push af
	call Draw_Hex_Byte
	pop af
	or a								;Filter for unprintable chars
	call z,Draw_Unprintable
	cp $ed
	call nc,Draw_Unprintable
	cp $d6
	call z,Draw_Unprintable_2
	jp _putmap							;then draw
Char_Copy:
	ld a,(hl)							;Get char that's currently selected
	ld hl,_undelBufLen
	inc (hl)							;increment length of string in undelete buffer
	ld c,(hl)
	ld b,0
	add hl,bc							;find the end of the buffer
	ld (hl),a							;and add our char!!!
Char_Refresh_Buf:
	ld hl,Undel_Buf_Pos					;Set cursor position
	ld (_penCol),hl
	ld hl,_undelBuf						;then display contents of undelete buffer
	ld a,(_undelBufLen)
	ld b,a
	or a								;If there aren't any chars, don't display any
	jp nz,_vputsn
	ret

Sys_Clean:								;Memory cleaning
;thanks to John Kempen for this code, which I optimized {
	ld hl,$ce2c						;clear entries
	ld ($ce2a),hl
	ld hl,0
	ld ($ceac),hl
;}
	ld a,Lx							;delete those idiotic stat vars
	call Delete_Var_3
	ld a,Ly
	call Delete_Var_3
	ld a,Lf
	call Delete_Var_3
	ld hl,ans_name-1				;kill ans
	call Delete_Var
	ld hl,eqn_name-1				;eqn
	call Delete_Var
	ld hl,regeq_name-1				;the undeletable regeq
	call Delete_Var
	ld hl,entry_name-1				;clear the "last entry" hidden prog
	push hl
	call Delete_Var
	pop hl
	rst 20h
	ld hl,0
	call _createprog
	call $4573						;now get rid of x and y-they don't do anyone
	call Delete_Var					;any good anyways
	call $4577
	call Delete_Var
	call _ram_page_1
	call Draw_Window				;Now inform user that vars have been deleted
	ld hl,Window_Pos
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,Vars_Deleted_Str
	call _vputs_newline
	call _vputs_newline
	call _vputs_newline
	call _vputs
	call Pause						;Wait for keypress
	jp Sys_Init						;then go back to sys mode

Delete_Var:							;delete var with name at (hl)
	rst 20h
Delete_Var_2:						;delete var with name in _OP1
	rst 10h
	ret c
	jp _delvar

Delete_Var_3:						;delete "stat" var with first letter in a
	ld hl,x_stat_name-1
	rst 20h
	ld (_OP1+2),a
	jr Delete_Var_2

;Port manager
Sys_Ports:
	ld a,stPort						;Set state
	ld (Cur_State),a
	ld hl,Sys_Ports					;Set address to return to if exit is pressed
	ld (Ret_Addr),hl
	call Clear_Wind					;Clear window
	ld hl,Port_Control_Pos			;Draw controls
	ld (_curRow),hl
	ld hl,Port_Control_Str
	call _puts
	ld hl,Port_In_Pos				;Draw "in" text
	ld (_penCol),hl
	ld hl,In_Str
	call _vputs
	jr Refresh_Port					;Draw port number and input from port

Port_Key_Handler:
	ld hl,Port_Num					;Init pointer
	ld a,(Key_Val)
	cp K_LEFT
	jr z,Port_Dec
	cp K_RIGHT
	jr z,Port_Inc
	cp K_SECOND
	ret nz
Port_Send:
	ld hl,Port_Prompt_Str			;Get num from user
	ld a,2
	ld b,a
	call Get_Hex_Num
	ld a,(Port_Num)
	ld c,a
	ld a,h
	out (c),a						;and send it to the selected port
	jr Sys_Ports
Port_Inc:
	inc (hl)						;Increment port number
	jr Refresh_Port
Port_Dec:
	dec (hl)						;Decrement port number
Refresh_Port:
	ld hl,Port_Pos					;Set large cursor back to position where port number will be drawn
	ld (_curRow),hl
	ld a,(Port_Num)					;Get port number
	push af
	call Draw_Hex_Byte_2			;and draw it in the appropriate place
	pop af
	ld c,a
	ld hl,Port_In_2_Pos				;Now position the cursor
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	in a,(c)						;read selected port
	push af
	call Draw_Hex_Byte				;and display the value - first in hex
	call _vnewline
	pop af
	call Draw_Bin_Byte				;then in binary
	ld a,(Port_Num)					;Now draw port description
	inc a
	cp 10							;Above port 7, we don't know what the heck they do
	ret nc
	ld hl,Port_Label_Table
	call Get_Addr_From_Table		;Otherwise, get the proper string
	ld de,Port_Label_Pos			;position the cursor
	ld (_penCol),de
	jp _vputs						;and display it

;Link com
Sys_Link:
	ld (iy+Link_Flags),0			;Clear flags
Sys_Link_2:
	ld a,stLink						;Set state
	ld (Cur_State),a
	ld hl,Sys_Link_2				;Set state to return to if exit pressed
	ld (Ret_Addr),hl
	call Clear_Wind					;Now clear window
	ld hl,Link_In_Pos				;and init cursor positions
	ld (Link_Rx_Pos),hl
	ld hl,Link_Out_Pos
	ld (_penCol),hl
	ld (Link_Tx_Pos),hl
	ld a,l
	ld (Patch2),a
	jp Draw_Link_Cur

Link_Port_Handler:
	ld d,Timeout_Num
	call ReceiveByte				;Get byte
	jp c,Link_Error					;If there was an error, show it
	ld hl,(Link_Rx_Pos)				;Otherwise, display received char
	ld (_penCol),hl
	bit bitHex,(iy+Link_Flags)		;Is it it hex mode?
	jr z,Link_Port_Disp_Char		;If so, display char
Link_Port_Disp_Hex:					;Otherwise, display hex
	push af
	ld a,l
	cp 128-10
	call nc,Link_Newline_Receive
	pop af
	call Draw_Hex_Byte
	ld a,Lspace
	call _vputmap
	jr Link_Port_Disp_Skip
Link_Port_Disp_Char:
	or a
	call z,Draw_Unprintable
	cp $e7
	call nc,Draw_Unprintable
	cp $d6
	call z,Draw_Unprintable_2
	call _vputmap
	ld a,l
	cp 128-7
	call nc,Link_Newline_Receive	;If at the end of the line, move to the next one
Link_Port_Disp_Skip:
	ld hl,(_penCol)					;Save cursor position
	ld (Link_Rx_Pos),hl
	ret

Link_Key_Handler:
	ld a,(Key_Val)
	cp K_ALPHA						;If it's the alpha key, then toggle alpha lock
	jr nz,Link_Test_More_Keys
	bit bitAlpha,(iy+Link_Flags)
	jr z,Link_Alpha_On
Link_Alpha_Off:
	res bitAlpha,(iy+Link_Flags)
	ret
Link_Alpha_On:
	set bitAlpha,(iy+Link_Flags)
	ret
Link_Test_More_Keys:
	cp K_SECOND						;If it's the second key, toggle hex mode
	jr nz,Link_Test_Even_More_Keys
	bit bitHex,(iy+Link_Flags)
	jr z,Link_Hex_On
Link_Hex_Off:
	res bitHex,(iy+Link_Flags)
	jp Sys_Link_2
Link_Hex_On:
	set bitHex,(iy+Link_Flags)
	jp Sys_Link_2
Link_Test_Even_More_Keys:
	ld hl,Key_Table					;Find the char equivalent of the key
	cp K_ALPHA
	ret nc
	sub K_PLUS
	ret c
	call Get_Addr_From_Table
	bit bitAlpha,(iy+Link_Flags)	;Is alpha mode on?
	jr z,Link_Send_Num				;If not, then send num value from table
Link_Send_Letter:					;If so, then send letter value for that key
	ld a,h
	jr Link_Send
Link_Send_Num:
	ld a,l
Link_Send:
	or a							;Is the selected char unavailable?
	ret z							;If so, return
	ld hl,(Link_Tx_Pos)				;Position cursor
	ld (_penCol),hl
	bit bitHex,(iy+Link_Flags)		;(if in hex mode, handle differently)
	jr z,Link_Send_Char
	bit bitNibble,(iy+Link_Flags)	;Is the user on the second nibble?
	jr nz,Link_Send_Hex_Last
	push af
	ld a,l							;If at end of line,
	cp 128-10
	call nc,Link_Newline_Send		;go to next one
	pop af
	set bitNibble,(iy+Link_Flags)
	ld (edit_buffer),a				;If not, store the first nibble
	call _vputmap
	ld hl,(_penCol)
	ld (Link_Tx_Pos),hl
	jr Draw_Link_Cur
Link_Send_Hex_Last:
	res bitNibble,(iy+Link_Flags)
	ld (edit_buffer+1),a			;If on second nibble, store it
	call _vputmap
	ld a,Lspace
	call _vputmap
	ld a,(_penCol)
	cp 128-7						;(if the cursor is at the end of the line,
	call nc,Link_Newline_Send		;move to the next one)
	ld hl,(_penCol)
	ld (Link_Tx_Pos),hl
	call Draw_Link_Cur
	ld a,(edit_buffer)
	call Convert_Char				;process whole byte
	call _shlacc
	ld b,a
	ld a,(edit_buffer+1)
	call Convert_Char
	or b
	jp SendByte						;and send it
Link_Send_Char:
	push af
	call _vputmap					;Draw the byte to be sent
	ld a,(_penCol)
	cp 128-7						;(if the cursor is at the end of the line,
	call nc,Link_Newline_Send		;move to the next one)
	pop af
	ld hl,(_penCol)
	ld (Link_Tx_Pos),hl
	push af
	call Draw_Link_Cur
	pop af
	jp SendByte						;Now send byte

Draw_Link_Cur:						;Draw link mode cursor
	set textInverse,(iy+textflags)
	bit bitHex,(iy+Link_Flags)		;What mode is it in?
	jr z,Draw_Link_Cur_C			;If it's in char mode, show that
Draw_Link_Cur_H:
	ld a,LcapH						;Otherwise, show that it's in hex mode
	jr Draw_Link_Cur_End
Draw_Link_Cur_C:
	ld a,LcapC
Draw_Link_Cur_End:
	call _vputmap
	res textInverse,(iy+textflags)
	ret

Link_Newline_Send:
	ld a,h
	cp Link_Out_Pos/256+((Link_Lines-1)*Line_Spacing)
	jp c,_vnewline
	;scroll
	ld hl,$fc00+((Link_Out_Pos/256+Line_Spacing)*16)
	ld de,$fc00+(Link_Out_Pos/256*16)
	ld bc,16*Link_Lines*Line_Spacing
	ldir
	ld a,(Patch2)
	ld (_penCol),a
	ret

Link_Newline_Receive:
	ld a,h
	cp Link_In_Pos/256+((Link_Lines-1)*Line_Spacing)
	jp c,_vnewline
	;scroll
	ld hl,$fc00+((Link_In_Pos/256+Line_Spacing)*16)
	ld de,$fc00+(Link_In_Pos/256*16)
	ld bc,16*(Link_Lines*Line_Spacing+1)
	ldir
	ld a,(Patch2)
	ld (_penCol),a
	ret

;Flags
Sys_Flags:
;	ld hl,Sys_Init					;Set to return to sys mode if exit pressed
;	ld (Ret_Addr),hl
Sys_Flags_1:
	call Clear_Wind					;Clear window
	call Refresh_Flags_1			;show state of flags
	ld hl,Text_Start_Pos			;set cursor position
	ld (_penCol),hl
	ld b,Flags_1_Menu_Num			;and show first flags menu
	ld hl,Flags_1_Menu_Text
	ld de,Flags_1_Menu_Table
	jp Do_Menu

Sys_Flags_2:
	call Clear_Wind					;Clear window
	call Refresh_Flags_2			;show state of flags
	ld hl,Text_Start_Pos			;set cursor position
	ld (_penCol),hl
	ld b,Flags_2_Menu_Num			;and show second flags menu
	ld hl,Flags_2_Menu_Text
	ld de,Flags_2_Menu_Table
	jp Do_Menu

Sys_Flags_3:
	call Clear_Wind					;Clear window
	call Refresh_Flags_3			;show state of flags
	ld hl,Text_Start_Pos			;set cursor position
	ld (_penCol),hl
	ld b,Flags_3_Menu_Num			;and show third flags menu
	ld hl,Flags_3_Menu_Text
	ld de,Flags_3_Menu_Table
	jp Do_Menu

Refresh_Flags_1:
	ld hl,Flag_Start_Pos			;Init cursor
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	bit apdable,(iy+apdflags)		;then test flags and display state
	call Draw_Bit_newline
	bit appTextSave,(iy+appflags)
	call Draw_Bit_newline
	bit curLock,(iy+curflags)
	call Draw_Bit_newline
	bit plotloc,(iy+plotflags)
	call Draw_Bit_newline
	bit alt_font,(iy+exceptionflg)
	call Draw_Bit_newline
	bit alt_vfont,(iy+exceptionflg)
	call Draw_Bit_newline
	bit alt_on,(iy+exceptionflg)
	jp Draw_Bit

Refresh_Flags_2:
	ld hl,Flag_Start_Pos
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	bit alt_off,(iy+exceptionflg)
	call Draw_Bit_newline
	bit alt_int,(iy+exceptionflg)
	call Draw_Bit_newline
	bit alt_link,(iy+exceptionflg)
	call Draw_Bit_newline
	bit alt_parse,(iy+exceptionflg2)
	call Draw_Bit_newline
	bit alt_form,(iy+exceptionflg2)
	call Draw_Bit_newline
	bit alt_exec,(iy+exceptionflg2)
	call Draw_Bit_newline
	bit alt_expr,(iy+exceptionflg2)
	jp Draw_Bit

Refresh_Flags_3:
	ld hl,Flag_Start_Pos
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	bit alt_home,(iy+exceptionflg2)
	call Draw_Bit_newline
	bit alt_cmdtok,(iy+exceptionflg2)
	call Draw_Bit_newline
	bit alt_key,(iy+exceptionflg2)
	call Draw_Bit_newline
	bit alt_grf,(iy+exceptionflg2)
	jp Draw_Bit

Flags_1_1:							;APD
	bit apdable,(iy+apdflags)
	jr z,Flags_1_1_On
Flags_1_1_Off:
	res apdable,(iy+apdflags)
	jp Refresh_Flags_1
Flags_1_1_On:
	set apdable,(iy+apdflags)
	jp Refresh_Flags_1

Flags_1_2:							;use _textShadow
	bit appTextSave,(iy+appflags)
	jr z,Flags_1_2_On
Flags_1_2_Off:
	res appTextSave,(iy+appflags)
	jp Refresh_Flags_1
Flags_1_2_On:
	set appTextSave,(iy+appflags)
	jp Refresh_Flags_1

Flags_1_3:							;cursor lock
	bit curLock,(iy+curflags)
	jr z,Flags_1_3_On
Flags_1_3_Off:
	res curLock,(iy+curflags)
	jp Refresh_Flags_1
Flags_1_3_On:
	set curLock,(iy+curflags)
	jp Refresh_Flags_1

Flags_1_4:							;use _plotSScreen
	bit plotloc,(iy+plotflags)
	jr z,Flags_1_4_On
Flags_1_4_Off:
	res plotloc,(iy+plotflags)
	jp Refresh_Flags_1
Flags_1_4_On:
	set plotloc,(iy+plotflags)
	jp Refresh_Flags_1

Flags_1_5:							;user large font
	bit alt_font,(iy+exceptionflg)
	jr z,Flags_1_5_On
Flags_1_5_Off:
	res alt_font,(iy+exceptionflg)
	jp Refresh_Flags_1
Flags_1_5_On:
	set alt_font,(iy+exceptionflg)
	jp Refresh_Flags_1

Flags_1_6:							;user small font
	bit alt_vfont,(iy+exceptionflg)
	jr z,Flags_1_6_On
Flags_1_6_Off:
	res alt_vfont,(iy+exceptionflg)
	jp Refresh_Flags_1
Flags_1_6_On:
	set alt_vfont,(iy+exceptionflg)
	jp Refresh_Flags_1

Flags_1_7:							;user on
	bit alt_on,(iy+exceptionflg)
	jr z,Flags_1_7_On
Flags_1_7_Off:
	res alt_on,(iy+exceptionflg)
	jp Refresh_Flags_1
Flags_1_7_On:
	set alt_on,(iy+exceptionflg)
	jp Refresh_Flags_1

Flags_2_1:							;user off
	bit alt_off,(iy+exceptionflg)
	jr z,Flags_2_1_On
Flags_2_1_Off:
	res alt_off,(iy+exceptionflg)
	jp Refresh_Flags_2
Flags_2_1_On:
	set alt_off,(iy+exceptionflg)
	jp Refresh_Flags_2

Flags_2_2:							;user interrupt
	bit alt_int,(iy+exceptionflg)
	jr z,Flags_2_2_On
Flags_2_2_Off:
	res alt_int,(iy+exceptionflg)
	jp Refresh_Flags_2
Flags_2_2_On:
	set alt_int,(iy+exceptionflg)
	jp Refresh_Flags_2

Flags_2_3:							;user link
	bit alt_link,(iy+exceptionflg)
	jr z,Flags_2_3_On
Flags_2_3_Off:
	res alt_link,(iy+exceptionflg)
	jp Refresh_Flags_2
Flags_2_3_On:
	set alt_link,(iy+exceptionflg)
	jp Refresh_Flags_2

Flags_2_4:							;sqrtParse
	bit alt_parse,(iy+exceptionflg2)
	jr z,Flags_2_4_On
Flags_2_4_Off:
	res alt_parse,(iy+exceptionflg2)
	jp Refresh_Flags_2
Flags_2_4_On:
	set alt_parse,(iy+exceptionflg2)
	jp Refresh_Flags_2

Flags_2_5:							;sqrtForm
	bit alt_form,(iy+exceptionflg2)
	jr z,Flags_2_5_On
Flags_2_5_Off:
	res alt_form,(iy+exceptionflg2)
	jp Refresh_Flags_2
Flags_2_5_On:
	set alt_form,(iy+exceptionflg2)
	jp Refresh_Flags_2

Flags_2_6:							;sqrtExec
	bit alt_exec,(iy+exceptionflg2)
	jr z,Flags_2_6_On
Flags_2_6_Off:
	res alt_exec,(iy+exceptionflg2)
	jp Refresh_Flags_2
Flags_2_6_On:
	set alt_exec,(iy+exceptionflg2)
	jp Refresh_Flags_2

Flags_2_7:							;sqrtExpr
	bit alt_expr,(iy+exceptionflg2)
	jr z,Flags_2_7_On
Flags_2_7_Off:
	res alt_expr,(iy+exceptionflg2)
	jp Refresh_Flags_2
Flags_2_7_On:
	set alt_expr,(iy+exceptionflg2)
	jp Refresh_Flags_2

Flags_3_1:							;sqrtHome
	bit alt_home,(iy+exceptionflg2)
	jr z,Flags_3_1_On
Flags_3_1_Off:
	res alt_home,(iy+exceptionflg2)
	jp Refresh_Flags_3
Flags_3_1_On:
	set alt_home,(iy+exceptionflg2)
	jp Refresh_Flags_3

Flags_3_2:							;sqrtCmdTok
	bit alt_cmdtok,(iy+exceptionflg2)
	jr z,Flags_3_2_On
Flags_3_2_Off:
	res alt_cmdtok,(iy+exceptionflg2)
	jp Refresh_Flags_3
Flags_3_2_On:
	set alt_cmdtok,(iy+exceptionflg2)
	jp Refresh_Flags_3

Flags_3_3:							;sqrtKey
	bit alt_key,(iy+exceptionflg2)
	jr z,Flags_3_3_On
Flags_3_3_Off:
	res alt_key,(iy+exceptionflg2)
	jp Refresh_Flags_3
Flags_3_3_On:
	set alt_key,(iy+exceptionflg2)
	jp Refresh_Flags_3

Flags_3_4:							;sqrtGrf
	bit alt_grf,(iy+exceptionflg2)
	jr z,Flags_3_4_On
Flags_3_4_Off:
	res alt_grf,(iy+exceptionflg2)
	jp Refresh_Flags_3
Flags_3_4_On:
	set alt_grf,(iy+exceptionflg2)
	jp Refresh_Flags_3

;VAT
Sys_VAT:
	ld hl,VAT_Addr_Stack			;Init addr stack
	ld (VAT_Addr_SP),hl
	ld hl,$bfff						;start at beginning of VAT
	ld (VAT_Top_Addr),hl
Sys_VAT_2:
	ld a,stVAT						;Show that is in VAT viewer mode
	ld (Cur_State),a
	ld hl,Sys_VAT_2
	ld (Ret_Addr),hl
	jr VAT_Refresh					;then draw vars

VAT_Key_Handler:
	ld a,(Key_Val)					;See what key was pressed
	cp K_DOWN
	jr z,VAT_Go_Down
	cp K_UP
	ret nz
VAT_Go_Up:
	ld de,(VAT_Addr_SP)
	ld hl,VAT_Addr_Stack
	call _cphlde					;Is it at the beginning of the stack?
	ret z							;If it is, we can't move up
	dec de							;Otherwise, pop address
	ld a,(de)
	ld h,a
	dec de
	ld a,(de)
	ld l,a
	ld (VAT_Top_Addr),hl			;and go to it
	ld (VAT_Addr_SP),de
	jr VAT_Refresh
VAT_Go_Down:
	ld de,(VAT_Bot_Addr)			;Are we at the end of the VAT?
	ld hl,($d298)
	call _cphlde
	ret z							;If so, don't allow user to go down any more
	ld hl,(VAT_Top_Addr)			;Otherwise, save VAT addr
	ld de,(VAT_Addr_SP)
	ld a,l
	ld (de),a
	inc de
	ld a,h
	ld (de),a
	inc de
	ld (VAT_Addr_SP),de
	ld hl,(VAT_Bot_Addr)			;then go to next page
	ld (VAT_Top_Addr),hl
VAT_Refresh:
	call Clear_Wind					;Clear window
	call _ram_page_7				;Load RAM page with VAT
	ld hl,Text_Start_Pos			;Set cursor position
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld b,Hex_Rows					;Set number of vars to draw
	ld hl,(VAT_Top_Addr)			;and get VAT pointer
VAT_Refresh_Loop:
	ld de,($d298)					;Get addr of end of VAT
	call _cphlde					;Are we at the end of it?
	jr z,VAT_Refresh_End			;If so, stop
	push bc
	push hl
	ld a,(hl)						;Get var type
	cp $14							;(if it's an unknown type, show that)
	jr nc,VAT_Refresh_Draw_NoType
	ld hl,Var_Type_Table			;and display it
	call Get_Addr_From_Table
	jr VAT_Refresh_Draw_Type
VAT_Refresh_Draw_NoType:
	ld hl,Unknown_Var_Str
VAT_Refresh_Draw_Type:
	call _vputs
	pop hl
	push hl
	call Disp_HL_Hex				;Now draw the VAT address of the var
	ld hl,Del_Str
	call _vputs						;(and put some spaces after it)
	pop hl
	dec hl							;Now look at abs addr of var
	ld e,(hl)						;(get the hl portion, and put in de)
	dec hl
	ld d,(hl)
	dec hl							;(get the a portion)
	ld a,(hl)
	push de
	call Draw_Hex_Byte				;Draw the first part
	pop de
	push hl
	push de
	pop hl
	call Disp_HL_Hex				;then the second
	ld hl,Del_Str
	call _vputs						;Put some space after it
	pop hl
	dec hl
	ld a,(hl)						;Draw flag byte
	call Draw_Hex_Byte
	push hl
	ld hl,Del_Str					;insert space after it
	call _vputs
	pop hl
	dec hl
	ld b,(hl)						;and get the name length
	dec hl
	ld a,b
	or a							;Is it 0?
	jr z,VAT_Refresh_End_Loop		;If so, skip drawing var name
VAT_Refresh_Name_Loop:				;and draw the var name
	ld a,(hl)
	call _vputmap
	dec hl
	djnz VAT_Refresh_Name_Loop
VAT_Refresh_End_Loop:
	call _vnewline
	pop bc
	djnz VAT_Refresh_Loop			;Loop
VAT_Refresh_End:
	ld (VAT_Bot_Addr),hl
	jp Reset_Pages					;Go back to RAM page 1 with data

;Stack viewer
Sys_Stack:
	ld a,stStack					;Init state
	ld (Cur_State),a
	ld hl,Sys_Stack
	ld (Ret_Addr),hl
	call Clear_Wind					;Clear window
	ld hl,Text_Start_Pos			;Position cursor
	push hl
	pop bc							;Save top-of-column position
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,$0002						;Get sp in hl,
	add hl,sp						;bypassing address of event handler
Stack_Loop:
	call Disp_HL_Hex				;First draw address sp is at
	ld a,Lcolon						;then separator
	call _vputmap
	push hl
	call _ldhlind					;Now get address at that stack level
	call Disp_HL_Hex				;draw it
	call _vnewline					;and go to the next line
	cp Hex_Bot+Line_Spacing			;Is it at the bottom of the line?
	jr nz,Stack_Loop_Continue
	push bc							;If so, move to next column
	pop hl
	ld a,l
	add a,Stack_Spacing
	ld l,a
	ld (_penCol),hl
	ld (Patch2),a
Stack_Loop_Continue:
	pop hl
	inc hl
	inc hl
	ld de,$fc00						;Are we at the end of the stack?
	call _cphlde
	jr nz,Stack_Loop				;If not, draw the next level
Stack_Key_Handler:
	ret								;If so, return

Sys_Getkey:
	ld a,stGetkey					;Set state
	ld (Cur_State),a
;	ld hl,Sys_Getkey
;	ld (Ret_Addr),hl
	res bitEnKey,(iy+Main_Flags)
Sys_Getkey_Init_Wind:
	call Clear_Wind					;Clear window
	ld hl,Text_Start_Pos			;and set cursor position
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,Getkey_Str
	jp _vputs_newline

Getkey_Key_Handler:
	ld a,(Key_Val)
	or a							;Has on been pressed?
	jp z,Ret_From_Keys
	ld ($c000),a					;Take current key
	set 3,(iy+0)					;indicate that it's been pressed
	call _getkey					;then get its _getkey code
	call Draw_Hex_Byte				;and display it
	ld a,Lspace
	call _vputmap
	ld a,(_penCol)					;Are we at the end of the line?
	cp 128-8
	ret c
	call _vnewline					;If so, go to the next line
	cp Hex_Bot+Line_Spacing			;Is the cursor at the bottom of the screen?
	ret nz
	jr Sys_Getkey_Init_Wind			;If so, clear window and start at the top again

Sys_Getky:
	ld a,stGetky					;Set state
	ld (Cur_State),a
;	ld hl,Sys_Getky
;	ld (Ret_Addr),hl
	res bitEnKey,(iy+Main_Flags)
Sys_Getky_Init_Wind:
	call Clear_Wind					;Clear window
	ld hl,Text_Start_Pos			;and set cursor position
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,Getky_Str
	jp _vputs_newline

Getky_Key_Handler:
	ld a,(Key_Val)					;Get keycode
	or a
	jr z,Ret_From_Keys				;If on was pressed,
	call Draw_Hex_Byte				;and display it
	ld a,Lspace
	call _vputmap
	ld a,(_penCol)					;Are we at the end of the line?
	cp 128-8
	ret c
	call _vnewline					;If so, go to the next line
	cp Hex_Bot+Line_Spacing			;Is the cursor at the bottom of the screen?
	ret nz
	jr Sys_Getky_Init_Wind			;If so, clear window and start at the top again

Sys_BASkey:
	ld a,stBASkey					;Set state
	ld (Cur_State),a
;	ld hl,Sys_BASkey
;	ld (Ret_Addr),hl
	res bitEnKey,(iy+Main_Flags)
Sys_BASkey_Init_Wind:
	call Clear_Wind					;Clear window
	ld hl,Text_Start_Pos			;and set cursor position
	ld (_penCol),hl
	ld a,l
	ld (Patch2),a
	ld hl,BASkey_Str
	jp _vputs_newline

BASkey_Key_Handler:
	ld a,(Key_Val)					;Has on been pressed?
	or a
	jr z,Ret_From_Keys				;If so, return
	call _op2toop1					;Otherwise, move keycode to op1
	ld a,3							;(set max length of string)
	call _formReal					;format it into a string
	ld hl,_OP3						;and display it
	call _vputs
	ld a,Lspace
	call _vputmap
	ld a,(_penCol)					;Are we at the end of the line?
	cp 128-12
	ret c
	call _vnewline					;If so, go to the next line
	cp Hex_Bot+Line_Spacing			;Is the cursor at the bottom of the screen?
	ret nz
	jr Sys_BASkey_Init_Wind			;If so, clear window and start at the top again

Ret_From_Keys:
	set bitEnKey,(iy+Main_Flags)
	jp Sys_2_Init

;///////////////////////////////////////////ERRORS//////////////////////////////////////////////////

Value_Error:
	ld hl,Value_Error_Str
	jr Error
Not_Found_Error:
	ld hl,Not_Found_Error_Str
	jr Error
Link_Error:
	ld hl,Link_Error_Str
Error:
	push hl
	call Draw_Window					;Draw window on screen
	pop hl
	ld de,Window_Pos					;position cursor
	ld (_penCol),de
	call _vputs							;Draw message
	call Pause							;Now wait for keypress
	pop hl
	ld hl,(Ret_Addr)
	jp Call_HL


#include "dis.asm"						;Include Dan Eble's disassembler routines
#include "link2.asm"					;Include link routines by Jimmy Mardell, Pascal Bouron, Joe Clussman
#include "textin.asm"					;Include text input routine by Cassady Roop
#include "LITE86.asm"					;Include Lite86 II decompressor by Kirk Meyer

#include "routines.asm"					;Include the other routines

Str_Name: .db $08,"syshackd"			;Name of string with all data

Var_Type_Table:					;Holds addresses of strings for var types
.dw Real_Var_Str
.dw Cplx_Var_Str
.dw RealVector_Var_Str
.dw CplxVector_Var_Str
.dw RealList_Var_Str
.dw CplxList_Var_Str
.dw RealMatrix_Var_Str
.dw CplxMatrix_Var_Str
.dw RealConst_Var_Str
.dw CplxConst_Var_Str
.dw Equ_Var_Str
.dw Unknown_Var_Str
.dw String_Var_Str
.dw GDB_Var_Str
.dw GDB_Var_Str
.dw GDB_Var_Str
.dw GDB_Var_Str
.dw Pic_Var_Str
.dw Prog_Var_Str
.dw Conv_Var_Str

Real_Var_Str: .db "Real       ",0
Cplx_Var_Str: .db "Cplx       ",0
RealVector_Var_Str: .db "Vect       ",0
CplxVector_Var_Str: .db "cVect   ",0
RealList_Var_Str: .db "List          ",0
CplxList_Var_Str: .db "cList      ",0
RealMatrix_Var_Str: .db "Mtrx     ",0
CplxMatrix_Var_Str: .db "cMtrx ",0
RealConst_Var_Str: .db "Const    ",0
CplxConst_Var_Str: .db "cConst",0
Equ_Var_Str: .db "Equ          ",0
Unknown_Var_Str: .db "?                  ",0
String_Var_Str: .db "Str           ",0
GDB_Var_Str: .db "GDB         ",0
Pic_Var_Str: .db "Pic            ",0
Prog_Var_Str: .db "Prgm    ",0
Conv_Var_Str: .db "Conv      ",0
Del_Str: .db "    ",0

ans_name: .db $03,"Ans"				;names of all the vars to delete
x_stat_name: .db $05,"xStat"
entry_name: .db $01,"#"
regeq_name: .db $05,"RegEq"
eqn_name: .db $03,"eqn"

.end